package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public class RET extends Instruction implements IndexedInstruction, TypedInstruction
{
    private boolean wide;
    private int index;
    
    RET() {
    }
    
    public RET(final int index) {
        super((short)169, (short)2);
        this.setIndex(index);
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitRET(this);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        if (this.wide) {
            out.writeByte(196);
        }
        out.writeByte(super.getOpcode());
        if (this.wide) {
            out.writeShort(this.index);
        }
        else {
            out.writeByte(this.index);
        }
    }
    
    @Override
    public final int getIndex() {
        return this.index;
    }
    
    @Override
    public Type getType(final ConstantPoolGen cp) {
        return ReturnaddressType.NO_TARGET;
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        this.wide = wide;
        if (wide) {
            this.index = bytes.readUnsignedShort();
            super.setLength(4);
        }
        else {
            this.index = bytes.readUnsignedByte();
            super.setLength(2);
        }
    }
    
    @Override
    public final void setIndex(final int n) {
        if (n < 0) {
            throw new ClassGenException("Negative index value: " + n);
        }
        this.index = n;
        this.setWide();
    }
    
    private void setWide() {
        this.wide = (this.index > 255);
        if (this.wide) {
            super.setLength(4);
        }
        else {
            super.setLength(2);
        }
    }
    
    @Override
    public String toString(final boolean verbose) {
        return super.toString(verbose) + " " + this.index;
    }
}
